package datavisualizer;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import java.io.IOException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.InputSource;

import java.io.FileInputStream;
import java.io.InputStream;
import com.ice.tar.TarInputStream;
import com.ice.tar.TarEntry;

import javax.swing.JInternalFrame;
import java.util.Vector;
import javawrapper.ModelException;
import javawrapper.ErrorGUI;

/**
 * Controls the data visualization for a single detailed output file.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.2
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>June 28, 2004:  Added support for snags (LEM)
 * <br>March 9, 2005:  Added line graph for volume and made version 1.1 (LEM)
 * <br>March 29, 2005:  Added histograms for grids and made version 1.2 (LEM)
 */

public class DetailedOutputFileManager
    extends DataFileManager {

  /**The collection of all data requests currently open*/
  private Vector mp_oDataRequests;

  /**Species names - the order determines the species codes*/
  private Vector mp_sSpeciesNames;

  /**Collection of javawrapper.DetailedTreeSettings objects*/
  private Vector mp_oTreeSettings;

  /**Collection of javawrapper.DetailedGridSettings objects*/
  private Vector mp_oGridSettings;

  /**Whether each histogram option is float (true) or int (false)*/
  private boolean[] mp_bHistogramIsFloat;

  /**Histogram names - so we can figure out which value in mp_bHistogramIsFloat
   * value to use*/
  private String[] mp_sHistogramNames;

  /**Display string of filename - should have the last 20 chars.*/
  private String m_sFileDisplayName;
  private XMLReader m_oParser; /**<File parser*/
  private String m_sTempRoot, /**<Root directory to put temp files*/
      m_sTempDir, /**<Where we actually extract to*/
      m_sFileRoot; /**<Root file name of all detailed output files*/
  private float m_fXPlotLength, /**<Plot X length*/
      m_fYPlotLength, /**<Plot Y length*/
      m_fNumYearsPerTimestep; /**<Number of years per timestep*/
  private int m_iNumTimesteps, /**<Number of actual timesteps of data*/
      m_iNumParFileTimesteps, /**<Number of timestes from the par file*/
      m_iNumTypes = 7, /**<Number of tree types*/
      m_iLastTimestepRead; /**<When updating charts, this is the file parsed*/

  /**
   * Sets the number of timesteps for the detailed output file parameter file.
   * @param i Number of timesteps.
   */
  public void SetParFileTimesteps(int i) {
    m_iNumParFileTimesteps = i;
  }

  /**
   * Gets the number of actual data timesteps contained in this detailed output
   * file (as opposed to how many the par file says there are).
   * @return int Number of timesteps for this detailed output file.
   */
  public int GetNumberOfActualTimesteps() {
    return m_iNumTimesteps;
  }

  /**
   * Handles the request to draw a chart for this file.  The request will be
   * passed on to an appropriate DataRequest object.  If the chart is already
   * open, the open chart will be returned.
   * @param sGraphName Name of chart to draw.
   * @return The drawn chart, or NULL if it could not be drawn.
   * @throws ModelException If anything goes wrong.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>June 29, 2004:  Added disturbance results grid (LEM)
   * <br>March 29, 2005:  Added grid histograms (LEM)
   */
  public JInternalFrame DrawChart(String sGraphName) throws ModelException {

    DataRequest oRequest = null;
    boolean bValidChart = false;

    //See if the chart wanted is already open
    int i;
    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (sGraphName.equals(oRequest.GetChartName())) {
        return oRequest.GetChart();
      }
    }

    if (sGraphName.equalsIgnoreCase("XY Tree Map")) {
      oRequest = new TreeMapDataRequest(this, sGraphName);
      bValidChart = true;
    }
    else if (sGraphName.endsWith("Histogram")) {
      //Find the index of the histogram
      int iIndex = 0;
      for (i = 0; i < mp_sHistogramNames.length; i++) {
        if (mp_sHistogramNames[i].equals(sGraphName)) {
          iIndex = i;
          break;
        }
      }
      //LEM 03-29-05
      if (sGraphName.startsWith("Grid")) {
        String sGridName = mp_sHistogramNames[i].substring(mp_sHistogramNames[i].
            indexOf(":") + 2, mp_sHistogramNames[i].indexOf(" - "));
        String sMemberName = mp_sHistogramNames[i].substring(mp_sHistogramNames[
            i].indexOf(" - ") + 3, mp_sHistogramNames[i].indexOf("Histogram") - 1);
        oRequest = new GridHistogramDataRequest(sMemberName,
                                            mp_bHistogramIsFloat[iIndex], this,
                                            sGraphName, sGridName);
      } else {
        oRequest = new HistogramDataRequest(sGraphName.substring(0,
            sGraphName.indexOf(" Histogram")),
                                            mp_bHistogramIsFloat[iIndex], this,
                                            sGraphName);
      }
      bValidChart = true;
    }
    else if (sGraphName.endsWith("Harvest Results") ||
             sGraphName.endsWith("Mortality Episode Results")) {
      oRequest = new HarvestDataRequest(sGraphName, this);
      bValidChart = true;
    }

    //Grid map object
    else if (sGraphName.startsWith("Map")) {

      //Parse out the grid name and data member name
      String sGridName = sGraphName.substring(sGraphName.indexOf(" - ") + 3,
                                              sGraphName.lastIndexOf(" - "));
      String sDataMember = sGraphName.substring(sGraphName.lastIndexOf(" - ") +
                                                3);
      oRequest = new GridDataRequest(sGraphName, sGridName, sDataMember, this);
      bValidChart = true;
    }

    //Tables
    else if (sGraphName.equals("Whole plot: Table")) {
      oRequest = new OverviewTableDataRequest(this, sGraphName);
      bValidChart = true;
    }
    else if (sGraphName.equals("BA Stand Table")) {
      oRequest = new StandTableDataRequest(this, sGraphName,
                                           StandTableDataRequest.BASAL_AREA);
      bValidChart = true;
    }
    else if (sGraphName.equals("Density Stand Table")) {
      oRequest = new StandTableDataRequest(this, sGraphName,
                                           StandTableDataRequest.DENSITY);
      bValidChart = true;
    }
    else if (sGraphName.equals("Stock Table")) {
      oRequest = new StockTableDataRequest(this, sGraphName);
      bValidChart = true;
    }

    //Line graphs
    else if (sGraphName.equals("Whole plot: Seedling absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.SEEDLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Seedling relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.SEEDLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.SAPLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.SAPLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling absolute basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          ABSOLUTE_BASAL_AREA,
                                          javawrapper.TreePopulation.SAPLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling relative basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          RELATIVE_BASAL_AREA,
                                          javawrapper.TreePopulation.SAPLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.ADULT);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.ADULT);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult absolute basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          ABSOLUTE_BASAL_AREA,
                                          javawrapper.TreePopulation.ADULT);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult relative basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          RELATIVE_BASAL_AREA,
                                          javawrapper.TreePopulation.ADULT);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.SNAG);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.SNAG);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag absolute basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          ABSOLUTE_BASAL_AREA,
                                          javawrapper.TreePopulation.SNAG);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag relative basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          RELATIVE_BASAL_AREA,
                                          javawrapper.TreePopulation.SNAG);
      bValidChart = true;
    }
    else if (sGraphName.equals("Sapling absolute volume")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          ABSOLUTE_VOLUME,
                                          javawrapper.TreePopulation.SAPLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Sapling relative volume")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          RELATIVE_VOLUME,
                                          javawrapper.TreePopulation.SAPLING);
      bValidChart = true;
    }
    else if (sGraphName.equals("Adult absolute volume")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          ABSOLUTE_VOLUME,
                                          javawrapper.TreePopulation.ADULT);
      bValidChart = true;
    }
    else if (sGraphName.equals("Adult relative volume")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          RELATIVE_VOLUME,
                                          javawrapper.TreePopulation.ADULT);
      bValidChart = true;
    }
    else if (sGraphName.equals("Snag absolute volume")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          ABSOLUTE_VOLUME,
                                          javawrapper.TreePopulation.SNAG);
      bValidChart = true;
    }
    else if (sGraphName.equals("Snag relative volume")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.
                                          RELATIVE_VOLUME,
                                          javawrapper.TreePopulation.SNAG);
      bValidChart = true;
    }

    if (bValidChart) {
      mp_oDataRequests.add(oRequest);

      //Parse our file
      ReadFile(m_iLastTimestepRead);
      return oRequest.DrawChart(m_oLegend,
                                sGraphName + " - " + m_sFileDisplayName);
    }
    else {
      return null;
    }
  }

  /**
   * Updates charts for a current run.  This re-inflates the tarball and gets
   * the last timestep file for parsing and processing.
   * @throws ModelException if there is a problem reading the file or
   * drawing the charts.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>November 18, 2004: Created (LEM)
   */
  public void UpdateCurrentRunCharts() throws ModelException {
    if (mp_oCharts.size() == 0) {
      return;
    }

    //Get the new number of timesteps
    CountTimesteps();

    //Extract the tarball again
    javawrapper.Tarball.ExtractTarball(m_sFilename, m_sTempRoot);

    //Advance through the legend
    DetailedOutputLegend oLegend = (DetailedOutputLegend) m_oLegend;
    oLegend.SetNumberOfTimesteps(m_iNumTimesteps);

    ( (DetailedOutputLegend) m_oLegend).GoToTimestep(m_iNumTimesteps);
  }

  /**
   * Gets the tables that can be drawn for this file.
   * @return String of tables, or NULL if no tables can be drawn.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>June 29, 2004:  Added disturbance table support (LEM)
   * <br>October 29, 2004:  Added stand and stock tables (LEM)
   */
  public String[] GetTableOptions() {

    Vector p_oOptions = new Vector(0);
    javawrapper.DetailedGridSettings oSettings;
    int i, j;

    //See if there's a harvest or disturbance results grid
    for (i = 0; i < mp_oGridSettings.size(); i++) {

      oSettings = (javawrapper.DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (oSettings.GetName().equals("Harvest Results")) {
        p_oOptions.add("Harvest Results");
      }
      else if (oSettings.GetName().equals("Mortality Episode Results")) {
        p_oOptions.add("Mortality Episode Results");
      }
    }

    //See if there are tree DBH or diam10 datamembers
    javawrapper.DetailedTreeSettings oTreeSetting;

    //Can we do diameter (overview table)?
    boolean bHasDiam = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("dbh")) {
          bHasDiam = true;
          break;
        }
        else if (sLabel.equalsIgnoreCase("diam10") &&
                 oTreeSetting.GetType() == javawrapper.TreePopulation.SEEDLING) {
          bHasDiam = true;
          break;
        }
      }
      if (bHasDiam) {
        p_oOptions.add("Whole plot: Table");
        break;
      }
    }

    //Can we do DBH (stand tables)?
    boolean bHasDBH = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("dbh")) {
          bHasDBH = true;
          break;
        }
      }
      if (bHasDBH) {
        p_oOptions.add("BA Stand Table");
        p_oOptions.add("Density Stand Table");
        break;
      }
    }

    //If we have volume, we can do a stock table
    boolean bHasVolume = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("volume")) {
          bHasVolume = true;
          break;
        }
      }
      if (bHasVolume) {
        p_oOptions.add("Stock Table");
        break;
      }
    }

    //Transfer the options list to an array
    if (p_oOptions.size() > 0) {
      String[] p_sOptions = new String[p_oOptions.size()];
      for (i = 0; i < p_sOptions.length; i++) {
        p_sOptions[i] = (String) p_oOptions.elementAt(i);
      }
      return p_sOptions;
    }
    return null;
  }

  /**
   * Gets the historgrams that can be drawn for this file.
   * @return String of tables, or NULL if no histograms can be drawn.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>March 29, 2005:  Added grid histograms (LEM)
   */
  public String[] GetHistogramOptions() {
    Vector oOptions = new Vector(0);
    javawrapper.DetailedTreeSettings oSetting;
    javawrapper.DetailedGridSettings oGridSetting;
    String sLabel, sTestLabel;
    int i, j, k;
    boolean bFound;

    for (i = 0; i < mp_oTreeSettings.size(); i++) {

      oSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.elementAt(
          i);

      //Go through the floats - we could histogram any of them
      for (j = 0; j < oSetting.GetNumberOfFloats(); j++) {

        //But we'll skip X and Y
        sLabel = oSetting.GetFloat(j).GetCodeName();
        if (!sLabel.equalsIgnoreCase("x") && !sLabel.equalsIgnoreCase("y")) {

          //Make sure we haven't already added this float
          bFound = false;
          for (k = 0; k < oOptions.size(); k++) {
            sTestLabel = (String) oOptions.elementAt(k);
            if (sLabel.equals(sTestLabel)) {
              bFound = true;
              break;
            }
          }
          if (!bFound) {
            oOptions.add(sLabel);
          }
        }
      }
      //Go through the ints - we could histogram any of them
      for (j = 0; j < oSetting.GetNumberOfInts(); j++) {

        sLabel = oSetting.GetInt(j).GetCodeName();

        //Make sure we haven't already added this int
        bFound = false;
        for (k = 0; k < oOptions.size(); k++) {
          sTestLabel = (String) oOptions.elementAt(k);
          if (sLabel.equals(sTestLabel)) {
            bFound = true;
            break;
          }
        }
        if (!bFound) {
          oOptions.add(sLabel);
        }
      }
    }

    //LEM 03-29-05
    for (i = 0; i < mp_oGridSettings.size(); i++) {

      oGridSetting = (javawrapper.DetailedGridSettings) mp_oGridSettings.
          elementAt(i);

      //Go through the floats - we could histogram any of them
      for (j = 0; j < oGridSetting.GetNumberOfFloats(); j++) {

        sLabel = oGridSetting.GetFloat(j).GetCodeName();

        //Make sure we haven't already added this float
        bFound = false;
        for (k = 0; k < oOptions.size(); k++) {
          sTestLabel = (String) oOptions.elementAt(k);
          if (sLabel.equals(sTestLabel)) {
            bFound = true;
            break;
          }
        }
        if (!bFound) {
          oOptions.add("Grid: " + oGridSetting.GetName() + " - " + sLabel);
        }
      }
      //Go through the ints - we could histogram any of them
      for (j = 0; j < oGridSetting.GetNumberOfInts(); j++) {

        sLabel = oGridSetting.GetInt(j).GetCodeName();

        //Make sure we haven't already added this int
        bFound = false;
        for (k = 0; k < oOptions.size(); k++) {
          sTestLabel = (String) oOptions.elementAt(k);
          if (sLabel.equals(sTestLabel)) {
            bFound = true;
            break;
          }
        }
        if (!bFound) {
          oOptions.add("Grid: " + oGridSetting.GetName() + " - " + sLabel);
        }
      }
    }

    //Transform our Vector into an array of strings
    if (oOptions.size() == 0) {
      return null;
    }
    mp_sHistogramNames = new String[oOptions.size()];
    for (i = 0; i < mp_sHistogramNames.length; i++) {
      mp_sHistogramNames[i] = (String) oOptions.elementAt(i);
    }

    //Get the float/int options
    mp_bHistogramIsFloat = new boolean[oOptions.size()];
    for (i = 0; i < mp_sHistogramNames.length; i++) {

      mp_bHistogramIsFloat[i] = false;

      if (mp_sHistogramNames[i].startsWith("Grid:") == false) {
        //Go through the floats - if we find a name match, set the histogram
        //to true
        for (k = 0; k < mp_oTreeSettings.size(); k++) {
          oSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.
              elementAt(
              k);
          for (j = 0; j < oSetting.GetNumberOfFloats(); j++) {

            sLabel = oSetting.GetFloat(j).GetCodeName();
            if (sLabel.equals(mp_sHistogramNames[i])) {
              mp_bHistogramIsFloat[i] = true;
              break;
            }
          }
          if (mp_bHistogramIsFloat[i]) {
            break;
          }
        }
      }
      else {
        //LEM -3/29/05
        String sGridName = mp_sHistogramNames[i].substring(mp_sHistogramNames[i].
            indexOf(":") + 2, mp_sHistogramNames[i].indexOf(" - "));
        String sMemberName = mp_sHistogramNames[i].substring(mp_sHistogramNames[
            i].indexOf(" - ") + 3);
        //Go through the grid floats - if we find a name match, set the histogram
        //to true
        for (k = 0; k < mp_oGridSettings.size(); k++) {
          oGridSetting = (javawrapper.DetailedGridSettings) mp_oGridSettings.
              elementAt(k);
          if (sGridName.equals(oGridSetting.GetName())) {
            for (j = 0; j < oGridSetting.GetNumberOfFloats(); j++) {

              sLabel = oGridSetting.GetFloat(j).GetCodeName();
              if (sLabel.equals(sMemberName)) {
                mp_bHistogramIsFloat[i] = true;
                break;
              }
            }
            if (mp_bHistogramIsFloat[i]) {
              break;
            }
          }
        }
      }
    }

    //Add the name "Histogram" to each name
    for (i = 0; i < mp_sHistogramNames.length; i++) {
      mp_sHistogramNames[i] = mp_sHistogramNames[i] + " Histogram";
    }

    return mp_sHistogramNames;
  }

  /**
   * Gets the maps that can be drawn for this file.
   * @return String of maps, or NULL if no maps can be drawn.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String[] GetMapOptions() {
    //This will hold all our options as we collect them
    Vector oOptions = new Vector(0);
    javawrapper.DetailedTreeSettings oTreeSetting;
    javawrapper.DetailedGridSettings oGridSetting;
    int i, j;

    //Can we do X, Y, and DBH?
    boolean bHasX = false, bHasY = false, bHasDBH = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("x")) {
          bHasX = true;
        }
        if (sLabel.equalsIgnoreCase("y")) {
          bHasY = true;
        }
        if (sLabel.equalsIgnoreCase("dbh")) {
          bHasDBH = true;
        }
      }

      if (bHasX && bHasY && bHasDBH) {
        oOptions.add("XY Tree Map");
        break;
      }

      else {
        bHasX = false;
        bHasY = false;
      }
    }

    //We can map any non-char, non-package grid data members
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oGridSetting = (javawrapper.DetailedGridSettings) mp_oGridSettings.
          elementAt(i);
      String sGridName = oGridSetting.GetName();
      for (j = 0; j < oGridSetting.GetNumberOfBools(); j++) {
        String sLabel = oGridSetting.GetBool(j).GetCodeName();
        oOptions.add("Map - " + sGridName + " - " + sLabel);
      }
      for (j = 0; j < oGridSetting.GetNumberOfInts(); j++) {
        String sLabel = oGridSetting.GetInt(j).GetCodeName();
        oOptions.add("Map - " + sGridName + " - " + sLabel);
      }
      for (j = 0; j < oGridSetting.GetNumberOfFloats(); j++) {
        String sLabel = oGridSetting.GetFloat(j).GetCodeName();
        oOptions.add("Map - " + sGridName + " - " + sLabel);
      }
    }

    if (oOptions.size() == 0) {
      return null;
    }

    String[] p_sOptions = new String[oOptions.size()];
    for (i = 0; i < p_sOptions.length; i++) {
      p_sOptions[i] = (String) oOptions.elementAt(i);
    }

    return p_sOptions;

  }

  /**
   * Gets the line graphs that can be drawn for this file.
   * @return String of tables, or NULL if no line graphs can be drawn.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>November 15, 2004:  Added line graph choices (LEM)
   * <br>March 9, 2005:  Added volume line graphs (LEM)
   */
  public String[] GetLineGraphOptions() {
    Vector p_oOptions = new Vector(0);
    javawrapper.DetailedTreeSettings oTreeSetting;
    int i, j;
    boolean bSeedling = false, bSaplingDiam = false, bAdultDiam = false,
        bSnagDiam = false, bSaplingVolume = false, bAdultVolume = false,
        bSnagVolume = false;

    //Check for which tree types we have diameter information
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (javawrapper.DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("dbh")) {
          if (oTreeSetting.GetType() == javawrapper.TreePopulation.SAPLING) {
            bSaplingDiam = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.ADULT) {
            bAdultDiam = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.SNAG) {
            bSnagDiam = true;
          }
        }
        else if (sLabel.equalsIgnoreCase("diam10") &&
                 oTreeSetting.GetType() == javawrapper.TreePopulation.SEEDLING) {
          bSeedling = true;
        }
        else if (sLabel.equalsIgnoreCase("volume")) {
          if (oTreeSetting.GetType() == javawrapper.TreePopulation.SAPLING) {
            bSaplingVolume = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.ADULT) {
            bAdultVolume = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.SNAG) {
            bSnagVolume = true;
          }
        }
      }
    }

    if (bSeedling) {
      p_oOptions.add("Whole plot: Seedling absolute density");
      p_oOptions.add("Whole plot: Seedling relative density");
    }
    if (bSaplingDiam) {
      p_oOptions.add("Whole plot: Sapling absolute basal area");
      p_oOptions.add("Whole plot: Sapling relative basal area");
      p_oOptions.add("Whole plot: Sapling absolute density");
      p_oOptions.add("Whole plot: Sapling relative density");
    }
    if (bAdultDiam) {
      p_oOptions.add("Whole plot: Adult absolute basal area");
      p_oOptions.add("Whole plot: Adult relative basal area");
      p_oOptions.add("Whole plot: Adult absolute density");
      p_oOptions.add("Whole plot: Adult relative density");
    }
    if (bSnagDiam) {
      p_oOptions.add("Whole plot: Snag absolute basal area");
      p_oOptions.add("Whole plot: Snag relative basal area");
      p_oOptions.add("Whole plot: Snag absolute density");
      p_oOptions.add("Whole plot: Snag relative density");
    }
    if (bSaplingVolume) {
      p_oOptions.add("Sapling absolute volume");
      p_oOptions.add("Sapling relative volume");
    }
    if (bAdultVolume) {
      p_oOptions.add("Adult absolute volume");
      p_oOptions.add("Adult relative volume");
    }
    if (bSnagVolume) {
      p_oOptions.add("Snag absolute volume");
      p_oOptions.add("Snag relative volume");
    }

    //Transfer the options list to an array
    if (p_oOptions.size() > 0) {
      String[] p_sOptions = new String[p_oOptions.size()];
      for (i = 0; i < p_sOptions.length; i++) {
        p_sOptions[i] = (String) p_oOptions.elementAt(i);
      }
      return p_sOptions;
    }
    return null;

  }

  /**
   * Sets the plot X length.
   * @param f Plot X length, in meters.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetXPlotLength(float f) {
    m_fXPlotLength = f;
  }

  /**
   * Sets the plot Y length.
   * @param f Plot Y length, in meters.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetYPlotLength(float f) {
    m_fYPlotLength = f;
  }

  /**
   * Sets the number of years per timestep.
   * @param f Number of years per timestep.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>November 2, 2004: Created (LEM)
   */
  public void SetNumberOfYearsPerTimestep(float f) {
    m_fNumYearsPerTimestep = f;
  }

  /**
   * Gets the number of years per timestep.
   * @return Number of years per timestep.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>November 2, 2004: Created (LEM)
   */
  public float GetNumberOfYearsPerTimestep() {
    return m_fNumYearsPerTimestep;
  }

  /**
   * Gets the plot X length.
   * @return Plot X length, in meters.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetXPlotLength() {
    return m_fXPlotLength;
  }

  /**
   * Gets the plot Y length.
   * @return Plot Y length, in meters.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetYPlotLength() {
    return m_fYPlotLength;
  }

  /**
   * Constructor.
   * @param sTarball  Detailed output file to manage.
   * @throws ModelException If there was a problem reading the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DetailedOutputFileManager(String sTarball) throws ModelException {
    super(sTarball);
    try {
      mp_oDataRequests = new Vector(0);
      mp_sSpeciesNames = new Vector(0);
      mp_oTreeSettings = new Vector(0);
      mp_oGridSettings = new Vector(0);

      m_iLastTimestepRead = 0;

      //Put together the root temp directory - the application's directory
      //plus temp
      String sPath = System.getProperty("SORTIE_PATH");
      String sSeparator = System.getProperty("file.separator");
      if (sPath == null) {
        m_sTempRoot = sSeparator + "temp";
      }
      else {
        if (sPath.endsWith(sSeparator) == false) {
          sPath += sSeparator;
        }
        m_sTempRoot = sPath + "temp";
      }

      //Create a display name
      if (sTarball.length() > 20) {
        //Take the LAST 20
        String sText = sTarball.substring(sTarball.length() - 20);
        m_sFileDisplayName = "..." + sText;
      }
      else {
        m_sFileDisplayName = sTarball;
      }

      //Get the root filename
      m_sFileRoot = sTarball.substring(0, sTarball.indexOf(".gz.tar"));

      m_sTempDir = javawrapper.Tarball.ExtractTarball(m_sFilename, m_sTempRoot);

      //Create our parser
      //Create the parser this way - don't go through the SAXParserFactory!
      //Parsers created that way throw MalformedURLExceptions when asked to
      //read local files.
      m_oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");

      ParseParamFile();
      CountTimesteps();

      //Create the legend
      String[] sSpeciesNames = new String[mp_sSpeciesNames.size()];
      for (int i = 0; i < mp_sSpeciesNames.size(); i++) {
        sSpeciesNames[i] = (String) mp_sSpeciesNames.elementAt(i);
      }

      String sFileName = new java.io.File(m_sFilename).getName();
      m_oLegend = new DetailedOutputLegend(this, sFileName, sSpeciesNames,
                                           m_iNumTimesteps);

    }
    catch (SAXException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.UNKNOWN;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = "There were problems constructing the XML parser.";
      throw (oErr);
    }
  }

  /**
   * Redraws all the charts currently open for this detailed output file.  This
   * is accomplished by sending an UpdateChart() call to each data request
   * object.
   * @throws ModelException wrapping another exception.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void UpdateCharts() throws ModelException {
    DataRequest oRequest;
    int i;
    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oRequest.UpdateChart(m_oLegend);
    }
  }

  /**
   * Adds a species name to the end of the species name list.  The position
   * within the vector becomes that name's species index code.
   * @param sName The species name to add.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddSpeciesName(String sName) {
    mp_sSpeciesNames.add(sName);
  }

  /**
   * Creates a new javawrapper.DetailedTreeSettings object and adds it to the
   * tree settings vector.
   * @param sSpeciesName The species name string
   * @param iType The type number
   * @return The new object created
   * @throws ModelException if either the type or the species name is invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public javawrapper.DetailedTreeSettings CreateNewTreeSettings(String
      sSpeciesName,
      int iType) throws
      ModelException {
    String sName;
    int iSpecies = -1, i;

    //Validate the type
    if (iType < 0 || iType >= m_iNumTypes) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage =
          "Invalid type encountered while parsing detailed output file - " +
          String.valueOf(iType);
      throw (oErr);
    }

    //Try to find the species
    for (i = 0; i < mp_sSpeciesNames.size(); i++) {
      sName = (String) mp_sSpeciesNames.elementAt(i);
      if (sName.equals(sSpeciesName)) {
        iSpecies = i;
        break;
      }
    }

    //Throw an error if we didn't find the name
    if (iSpecies == -1) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage =
          "Invalid species name encountered while parsing detailed output file - " +
          sSpeciesName;
      throw (oErr);
    }

    javawrapper.DetailedTreeSettings oData = new javawrapper.
        DetailedTreeSettings(
        iType, iSpecies);

    mp_oTreeSettings.add(oData);

    return oData;
  }

  /**
   * Creates a new javawrapper.DetailedGridSettings object and adds it to the
   * grid settings vector.
   * @param sGridName The grid name string
   * @return The new object created
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public javawrapper.DetailedGridSettings CreateNewGridSettings(String
      sGridName) {

    javawrapper.DetailedGridSettings oData = new javawrapper.
        DetailedGridSettings(
        sGridName);

    mp_oGridSettings.add(oData);

    return oData;
  }

  /**
   * Adds the X cell length to grid settings.
   * @param sGridName The grid name for which to add the cell length.
   * @param fLength The new X cell length in m.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridXCellLength(String sGridName, float fLength) {

    //Go through our grids and find this one
    javawrapper.DetailedGridSettings oSettings;
    int i;
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (javawrapper.DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (oSettings.GetName().equals(sGridName)) {
        oSettings.SetXCellLength(fLength);
      }
    }
  }

  /**
   * Adds the Y cell length to grid settings.
   * @param sGridName The grid name for which to add the cell length.
   * @param fLength The new Y cell length in m.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridYCellLength(String sGridName, float fLength) {

    //Go through our grids and find this one
    javawrapper.DetailedGridSettings oSettings;
    int i;
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (javawrapper.DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (oSettings.GetName().equals(sGridName)) {
        oSettings.SetYCellLength(fLength);
      }
    }
  }

  /**
   * Counts the number of timesteps in the tarball and places it in
   * m_iNumTimesteps.  The value is found by counting the number of
   * files.  The number of timesteps is the number of files - 2 (for the
   * parameter file and initial conditions).  This is compared to the
   * number of timesteps found in the header file and the lesser one is chosen.
   * @throws ModelException - wrapping other exceptions
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  private void CountTimesteps() throws ModelException {
    FileInputStream oFileStream = null;
    TarInputStream oTarballStream = null;
    try {
      TarEntry oTarEntry;

      int iCountedTimesteps;

      //Count the number of files in the tarball - it should be number of
      //timesteps plus 2 (for detailed output header and initial conditions)
      iCountedTimesteps = 0;

      oFileStream = new FileInputStream(m_sFilename);
      oTarballStream = new TarInputStream(oFileStream);
      oTarEntry = oTarballStream.getNextEntry();

      while (oTarEntry != null) {
        iCountedTimesteps++;
        oTarEntry = oTarballStream.getNextEntry();
      }

      if (iCountedTimesteps < m_iNumParFileTimesteps + 2) {
        m_iNumTimesteps = iCountedTimesteps - 2;
      }
      else {
        m_iNumTimesteps = m_iNumParFileTimesteps;
      }

    }
    catch (IOException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_FILE;
      oErr.sCallingFunction = "DoDetailedOutputSetup";
      oErr.sMessage = m_sFilename;
      throw (oErr);
    }
    finally {
      try {
        if (null != oTarballStream) {
          oTarballStream.close();
        }
        if (null != oFileStream) {
          oFileStream.close();
        }
      }
      catch (java.io.IOException e) {
        ; //do nothing
      }
    }
  }

  /**
   * Parses the parameter file portion of the tarball.  This function parses
   * the detailed output header file to find species information and tree setup
   * information.
   * @throws ModelException if the file cannot be parsed.
   */
  private void ParseParamFile() throws ModelException {
    InputStream oInputStream = null;
    String sFileToRead = "";
    try {
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new DetailedOutputFileSetupParseHandler(this);
      m_oParser.setContentHandler(oHandler);

      //Create the string which has the filename of the detailed output file
      sFileToRead = m_sFileRoot + ".xml.gz";
      //Trim off path and add temp directory
      sFileToRead = sFileToRead.substring(sFileToRead.lastIndexOf("\\") + 1);
      sFileToRead = m_sTempDir + sFileToRead;

      //Parse the file
      oInputStream = javawrapper.Tarball.UnzipFile(sFileToRead);
      InputSource oToParse = new InputSource(oInputStream);

      m_oParser.parse(oToParse);
    }
    catch (IOException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_FILE;
      oErr.sCallingFunction = "DoDetailedOutputSetup";
      oErr.sMessage = sFileToRead;
      throw (oErr);

    }
    catch (SAXException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_XML_FILE;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = e.getMessage();
      throw (oErr);
    }
    finally {
      if (oInputStream != null) {
        try {
          oInputStream.close();
        }
        catch (java.io.IOException e) {
          ; //do nothing
        }
      }
    }
  }

  /**
   * Gets the number of species that exist in the detailed output header file.
   * @return The number of species.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfSpecies() {
    return mp_sSpeciesNames.size();
  }

  /**
   * Gets the number of tree types.
   * @return The number of types.
   */
  public int GetNumberOfTypes() {
    return m_iNumTypes;
  }

  /**
   * When passed the name of a species, will give back the corresponding
   * species number code.
   * @param sName The name of the species.
   * @return The code.
   * @throws ModelException if the name is not recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetSpeciesCodeFromName(String sName) throws ModelException {
    int i;

    for (i = 0; i < mp_sSpeciesNames.size(); i++) {
      if (sName.equalsIgnoreCase( (String) mp_sSpeciesNames.elementAt(i))) {
        return i;
      }
    }
    ModelException oErr = new ModelException();
    oErr.iErrorCode = ErrorGUI.BAD_ARGUMENT;
    oErr.sCallingFunction = "JAVA";
    oErr.sMessage = "Unrecognized species " + sName +
        " passed to detailed output file manager for " + m_sFilename;
    throw (oErr);
  }

  /**
   * When passed the index of a species, will give back the corresponding
   * species name.
   * @param iIndex Species index.
   * @return Species name.
   * @throws ModelException if the index is unrecognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetSpeciesNameFromCode(int iIndex) throws ModelException {
    if (iIndex < 0 || iIndex >= mp_sSpeciesNames.size()) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_ARGUMENT;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = "Unrecognized species index" + String.valueOf(iIndex) +
          " passed to detailed output file manager for " + m_sFilename;
      throw (oErr);
    }

    return (String) mp_sSpeciesNames.elementAt(iIndex);
  }

  /**
   * Read the detailed output file for a particular timestep.
   * @param iTimestep Timestep for which to read the file.
   * @throws ModelException If the file cannot be read.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ReadFile(int iTimestep) throws ModelException {
    InputStream oInputStream = null;
    String sFileToRead = "";
    try {

      m_iLastTimestepRead = iTimestep;

      //Tell the open data requests to clear all their old data -
      //and remove any requests that no longer have charts open
      DataRequest oDataRequest;
      int i;
      for (i = 0; i < mp_oDataRequests.size(); i++) {
        oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
        if (oDataRequest.m_oChartFrame != null &&
            oDataRequest.m_oChartFrame.isVisible() == false) {
          mp_oDataRequests.remove(i);
          i--;
        }
        else {
          oDataRequest.ClearData();
        }
      }

      //Create the string which has the filename of the detailed output file
      sFileToRead = m_sFileRoot + "_" + String.valueOf(iTimestep) + ".xml.gz";
      //Trim off path and add temp directory
      sFileToRead = sFileToRead.substring(sFileToRead.lastIndexOf("\\") + 1);
      sFileToRead = m_sTempDir + sFileToRead;

      //Now install the detailed output timestep handler
      DefaultHandler oHandler = new DetailedOutputTimestepParseHandler(this);
      m_oParser.setContentHandler(oHandler);

      //Parse the file
      oInputStream = javawrapper.Tarball.UnzipFile(sFileToRead);
      InputSource oToParse = new InputSource(oInputStream);

      m_oParser.parse(oToParse);

    }
    catch (SAXException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_XML_FILE;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = "There were problems parsing " + sFileToRead;
      throw (oErr);
    }
    catch (IOException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_FILE;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = sFileToRead;
      throw (oErr);
    }
    finally {
      if (oInputStream != null) {
        try {
          oInputStream.close();
        }
        catch (java.io.IOException e) {
          ; //do nothing
        }
      }
    }

  }

  /**
   * Whether or not any open data requests want any tree float data members.
   * @return True if float data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyTreeFloats() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeFloats();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid float data members.
   * @return True if float data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyGridFloats() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridFloats();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any tree int data members.
   * @return True if int data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyTreeInts() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeInts();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid int data members.
   * @return True if int data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyGridInts() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridInts();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any tree char data members.
   * @return True if char data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyTreeChars() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeChars();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid char data members.
   * @return True if char data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyGridChars() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridChars();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any tree bool data members.
   * @return True if bool data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyTreeBools() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeBools();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid bool data members.
   * @return True if bool data members are desired, false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyGridBools() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridBools();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Accepts a piece of tree float data from the parser and passes it on to
   * open data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param fVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeFloatData(int iSpecies, int iType, int iCode, float fVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeFloats()) {
        oDataRequest.AddTreeFloatData(iSpecies, iType, iCode, fVal);
      }
    }
  }

  /**
   * Accepts a piece of grid float data from the parser and passes it on to
   * open data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param fVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridFloatData(String sGridName, int iX, int iY, int iCode,
                               float fVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridFloats()) {
        oDataRequest.AddGridFloatData(sGridName, iX, iY, iCode, fVal);
      }
    }
  }

  /**
   * Accepts a piece of tree int data from the parser and passes it on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param iVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeIntData(int iSpecies, int iType, int iCode, int iVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeInts()) {
        oDataRequest.AddTreeIntData(iSpecies, iType, iCode, iVal);
      }
    }
  }

  /**
   * Accepts a piece of grid int data from the parser and passes it on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param iVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridIntData(String sGridName, int iX, int iY, int iCode,
                             int iVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridInts()) {
        oDataRequest.AddGridIntData(sGridName, iX, iY, iCode, iVal);
      }
    }
  }

  /**
   * Accepts a piece of tree char data from the parser and passes it on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param sVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeCharData(int iSpecies, int iType, int iCode, String sVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeChars()) {
        oDataRequest.AddTreeCharData(iSpecies, iType, iCode, sVal);
      }
    }
  }

  /**
   * Accepts a piece of grid char data from the parser and passes it on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param sVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridCharData(String sGridName, int iX, int iY, int iCode,
                              String sVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridChars()) {
        oDataRequest.AddGridCharData(sGridName, iX, iY, iCode, sVal);
      }
    }
  }

  /**
   * Accepts a piece of tree bool data from the parser to pass on to open data
   * requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value and passes it on to open
   * data requests.
   * @param bVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeBoolData(int iSpecies, int iType, int iCode, boolean bVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeBools()) {
        oDataRequest.AddTreeBoolData(iSpecies, iType, iCode, bVal);
      }
    }
  }

  /**
   * Accepts a piece of grid bool data from the parser to pass on to open data
   * requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value and passes it on to open
   * data requests.
   * @param bVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridBoolData(String sGridName, int iX, int iY, int iCode,
                              boolean bVal) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridBools()) {
        oDataRequest.AddGridBoolData(sGridName, iX, iY, iCode, bVal);
      }
    }
  }

  /**
   * Accepts a tree float data member code from the parser to pass on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeFloatDataMemberCode(int iSpecies, int iType, String sLabel,
                                         int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeFloatDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid float data member code from the parser to pass on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridFloatDataMemberCode(String sGridName, String sLabel,
                                         int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridFloatDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts an int data member code from the parser to pass on to open data
   * requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeIntDataMemberCode(int iSpecies, int iType, String sLabel,
                                       int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeIntDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts an int data member code from the parser to pass on to open data
   * requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridIntDataMemberCode(String sGridName, String sLabel,
                                       int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridIntDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a tree char data member code from the parser to pass on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeCharDataMemberCode(int iSpecies, int iType, String sLabel,
                                        int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeCharDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid char data member code from the parser to pass on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridCharDataMemberCode(String sGridName, String sLabel,
                                        int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridCharDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a tree bool data member code from the parser to pass on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddTreeBoolDataMemberCode(int iSpecies, int iType, String sLabel,
                                        int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeBoolDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts a tree bool data member code from the parser to pass on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridBoolDataMemberCode(String sGridName, String sLabel,
                                        int iCode) {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridBoolDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Gets the length of X cells for a grid.
   * @param sGridName Grid for which to get the X cell length.
   * @return The X cell length, in m, or 0 if the grid is not recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetGridXCellLength(String sGridName) {
    javawrapper.DetailedGridSettings oSettings;
    float fLength = 0;
    int i;

    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (javawrapper.DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (sGridName.equals(oSettings.GetName())) {
        fLength = oSettings.GetXCellLength();
      }
    }
    return fLength;
  }

  /**
   * Gets the length of Y cells for a grid.
   * @param sGridName Grid for which to get the Y cell length.
   * @return The Y cell length, in m, or 0 if the grid is not recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetGridYCellLength(String sGridName) {
    javawrapper.DetailedGridSettings oSettings;
    float fLength = 0;
    int i;

    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (javawrapper.DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (sGridName.equals(oSettings.GetName())) {
        fLength = oSettings.GetYCellLength();
      }
    }
    return fLength;
  }

  /**
   * Deletes all files from the tarball in the temp folder
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void CleanUp() {
    javawrapper.Tarball.CleanUp(m_sFilename, m_sTempDir, m_sTempRoot);
  }
}
