package datavisualizer;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import javawrapper.ModelException;

/**
 * An object of this class will parse detailed output timestep files and pass
 * the data to a DetailedOutputFileManager object.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>March 21, 2005:  Fixed bug so we can differentiate between package and
 * non-package grid data members (LEM)
 */

public class DetailedOutputTimestepParseHandler
    extends DefaultHandler {

  private DetailedOutputFileManager m_oData;
      /**<Parent detailed output file manager.*/

  private StringBuffer m_sBuf;

  /**<String buffer for collecting character data*/

  private String m_sDataMemberLabel, /**<Data member label*/
      m_sGridName = ""; /**<Grid name*/
  private int m_iSpecies, /**<Tree species*/
      m_iType, /**<Tree type*/
      m_iDataMemberCode, /**<Data member code*/
      m_iX, /**<Grid cell X number*/
      m_iY; /**<Grid cell Y number*/

  //These are flags which are set by startElement so we know what to do when
  //we get the character feed from characters.
  private boolean
      /**Indicates the data just parsed is the X length of grid cells*/
      m_bIsGridXCellLength,
      /**Indicates the data just parsed is the Y length of grid cells*/
      m_bIsGridYCellLength,
      /**Indicates the data just parsed is the code for a float data member.*/
      m_bIsFloatCode,
      /**Indicates the data just parsed is the code for an int data member.*/
      m_bIsIntCode,
      /**Indicates the data just parsed is the code for a char data member.*/
      m_bIsCharCode,
      /**Indicates the data just parsed is the code for a bool data member.*/
      m_bIsBoolCode,
      /**Indicates the data just parsed is float data.*/
      m_bIsFloatData,
      /**<Indicates the data just parsed is int data.*/
      m_bIsIntData,
      /**<Indicates the data just parsed is char data.*/
      m_bIsCharData,
      /**<Indicates the data just parsed is bool data.*/
      m_bIsBoolData,
      /**Indicates whether any other object wants tree float values.*/
      m_bWantsTreeFloat,
      /**Indicates whether any other object wants tree int values.*/
      m_bWantsTreeInt,
      /**Indicates whether any other object wants tree char values.*/
      m_bWantsTreeChar,
      /**Indicates whether any other object wants tree bool values.*/
      m_bWantsTreeBool,
      /**Indicates whether any other object wants grid float values.*/
      m_bWantsGridFloat,
      /**Indicates whether any other object wants grid int values.*/
      m_bWantsGridInt,
      /**Indicates whether any other object wants grid char values.*/
      m_bWantsGridChar,
      /**Indicates whether any other object wants grid bool values.*/
      m_bWantsGridBool,
      /**Indicates that these are grid package values*/
      m_bIsPackage;

  /**
   * Constructor
   * @param oData Parent detailed output file manager.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DetailedOutputTimestepParseHandler(DetailedOutputFileManager oData) {
    m_oData = oData;

    //Set all our flags
    m_bIsFloatCode = false;
    m_bIsIntCode = false;
    m_bIsCharCode = false;
    m_bIsBoolCode = false;

    m_bIsFloatData = false;
    m_bIsIntData = false;
    m_bIsCharData = false;
    m_bIsBoolData = false;

    m_bIsGridXCellLength = false;
    m_bIsGridYCellLength = false;

    m_bIsPackage = false; //LEM 03-21-05

    m_sDataMemberLabel = "";

    //Set our shortcut flags
    m_bWantsTreeFloat = m_oData.WantAnyTreeFloats();
    m_bWantsTreeInt = m_oData.WantAnyTreeInts();
    m_bWantsTreeChar = m_oData.WantAnyTreeChars();
    m_bWantsTreeBool = m_oData.WantAnyTreeBools();

    m_bWantsGridFloat = m_oData.WantAnyGridFloats();
    m_bWantsGridInt = m_oData.WantAnyGridInts();
    m_bWantsGridChar = m_oData.WantAnyGridChars();
    m_bWantsGridBool = m_oData.WantAnyGridBools();
  }

  /**
   * Called when the parser hits a new opening tag.  This is overridden from
   * the base class.  This function is interested in the following tags:
   * <p>
   * Tree settings information:
   * <ul>
   * <li>tm_treeSettings - the species and type are saved.
   * <li>tm_floatCode - the label is extracted and m_bIsFloatCode and
   * m_bIsSettings are set to true.
   * <li>tm_intCode - the label is extracted and m_bIsIntCode and
   * m_bIsSettings are set to true.
   * <li>tm_charCode - the label is extracted and m_bIsCharCode and
   * m_bIsSettings are set to true.
   * <li>tm_boolCode - the label is extracted and m_bIsBoolCode and
   * m_bIsSettings are set to true.
   * </ul>
   * Tree data:
   * <ul>
   * <li>tree - the species and type is extracted and set in m_iSpecies and
   * m_iType.
   * <li>fl - m_bWantsTreeFloat is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>int - m_bWantsTreeInt is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>ch - m_bWantsTreeChar is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>bl - m_bWantsTreeBool is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * </ul>
   *
   * Grid settings information:
   * <ul>
   * <li>grid - m_sGridName is set.</li>
   * <li>ma_floatCode - the label is extracted and m_bIsFloatCode and
   * m_bIsSettings are set to true.</li>
   * <li>ma_intCode - the label is extracted and m_bIsIntCode and
   * m_bIsSettings are set to true.</li>
   * <li>ma_charCode - the label is extracted and m_bIsCharCode and
   * m_bIsSettings are set to true.</li>
   * <li>ma_boolCode - the label is extracted and m_bIsBoolCode and
   * m_bIsSettings are set to true.</li>
   * <li>Something starting with "ma_package" - m_bIsPackage is set to
   * true.</li>
   * </ul>
   * <p>
   * Grid data:
   * <ul>
   * <li>ma_v - m_iX and m_iY is extracted from the attributes.
   * <li>fl - m_bWantsTreeFloat is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>int - m_bWantsTreeInt is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>ch - m_bWantsTreeChar is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>bl - m_bWantsTreeBool is checked.  If true, the data member
   * code is extracted from the attributes and placed in m_sDataMemberLabel.
   * <li>ma_lengthXCells - m_bIsGridXCellLength is set to true.
   * <li>ma_lengthYCells - m_bIsGridYCellLength is set to true.
   * </ul>
   * @param sURI the Namespace URI (ignored)
   * @param sLocalName the local name (what this function looks at)
   * @param sQName the qualified (prefixed) name (ignored)
   * @param oAttributes The tag's oAttributes
   * @throws SAXException if there are any problems.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>March 21, 2005:  Fixed a bug so that package members could be
   * differentiated (LEM)
   */
  public void startElement(java.lang.String sURI,
                           java.lang.String sLocalName,
                           java.lang.String sQName,
                           Attributes oAttributes) throws SAXException {

    try {

      m_sBuf = null;
      m_sBuf = new StringBuffer();

      if (m_sGridName.length() == 0) {

        //*******************************************************
         //This is tree data
         //*******************************************************

          //Do the appropriate thing, depending on the tag name
          if (sQName.equals("fl") && m_bWantsTreeFloat) {
            //Set the float codes flag to true
            m_bIsFloatData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("int") && m_bWantsTreeInt) {
            //Set the int codes flag to true
            m_bIsIntData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("ch") && m_bWantsTreeChar) {
            //Set the char codes flag to true
            m_bIsCharData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("bl") && m_bWantsTreeBool) {
            //Set the bool codes flag to true
            m_bIsBoolData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("tree")) {
            //Get the type number and species name
            m_iType = new Integer(oAttributes.getValue("tp")).intValue();
            m_iSpecies = new Integer(oAttributes.getValue("sp")).intValue();
          } //Settings data
          else if (sQName.equals("tm_floatCode")) {

            m_bIsFloatCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("tm_intCode")) {

            m_bIsIntCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("tm_charCode")) {

            m_bIsCharCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("tm_boolCode")) {

            m_bIsBoolCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("tm_treeSettings")) {

            //Get the type and species
            String sSpecies = oAttributes.getValue("sp");
            m_iSpecies = m_oData.GetSpeciesCodeFromName(sSpecies);

            m_iType = new Integer(oAttributes.getValue("tp")).intValue();
          }

          //OR, this is the first grid tag
          else if (sQName.equals("grid")) {

            //Get the grid name
            m_sGridName = oAttributes.getValue("gridName");
          }

      }
      else {

        //*******************************************************
         //This is grid data
         //*******************************************************

          //Do the appropriate thing, depending on the tag name
          if (sQName.equals("fl") && m_bWantsGridFloat) {
            //Set the float codes flag to true
            m_bIsFloatData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("int") && m_bWantsGridInt) {
            //Set the int codes flag to true
            m_bIsIntData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("ma_v")) {

            m_iX = new Integer(oAttributes.getValue("x")).intValue();

            m_iY = new Integer(oAttributes.getValue("y")).intValue();
          }
          else if (sQName.equals("ma_floatCode")) {

            m_bIsFloatCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("ma_intCode")) {

            m_bIsIntCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.startsWith("ma_package")) { //LEM 03-25-05
            m_bIsPackage = true;
          }
          else if (sQName.equals("grid")) {

            //Get the grid name
            m_sGridName = oAttributes.getValue("gridName");
          }
          else if (sQName.equals("ch") && m_bWantsGridChar) {
            //Set the char codes flag to true
            m_bIsCharData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("bl") && m_bWantsGridBool) {
            //Set the bool codes flag to true
            m_bIsBoolData = true;
            //Grab the attribute which has the label and put it in m_iDataMemberCode
            String sCode = oAttributes.getValue("c");
            m_iDataMemberCode = new Integer(sCode).intValue();
          }
          else if (sQName.equals("ma_charCode")) {

            m_bIsCharCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("ma_boolCode")) {

            m_bIsBoolCode = true;

            //Get the label
            m_sDataMemberLabel = oAttributes.getValue("label");

          }
          else if (sQName.equals("ma_lengthXCells")) {

            m_bIsGridXCellLength = true;

          }
          else if (sQName.equals("ma_lengthYCells")) {

            m_bIsGridYCellLength = true;
          }

      }

      //Any other tags - we don't care.
    }
    catch (ModelException oErr) {
      //Wrap this in a SAXException
      SAXException e = new SAXException(oErr.sMessage);
      throw (e);
    }
  }

  /**
   * Feeds accumulated character data.  Whether or not the data is
   * ignored, and what is done with it if it is not ignored, depends on flags
   * that have been set.
   * @param sURI the Namespace URI (ignored)
   * @param sLocalName the local name (what this function looks at)
   * @param sQName the qualified (prefixed) name (ignored)
   * @throws SAXException if there are any problems.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>March 21, 2005:  Fixed a bug so that package members could be
   * differentiated (LEM)
   */
  public void endElement(java.lang.String sURI,
                         java.lang.String sLocalName,
                         java.lang.String sQName) throws SAXException {

    if (m_sBuf.length() == 0) {
      //Clear grid name
      if (sLocalName.equals("grid")) {
        m_sGridName = "";
      }
      return;
    }

    if (m_sGridName.length() == 0) {

      //*******************************************************
       //This is tree data
       //*******************************************************

        //Check our flags to see what we want to do
        if (m_bIsFloatData) {
          if (m_bWantsTreeFloat) {
            m_oData.AddTreeFloatData(m_iSpecies, m_iType, m_iDataMemberCode,
                                     new Float(m_sBuf.toString()).floatValue());
          }

          //Reset our flag
          m_bIsFloatData = false;

        }
        else if (m_bIsIntData) {
          if (m_bWantsTreeInt) {

            m_oData.AddTreeIntData(m_iSpecies, m_iType, m_iDataMemberCode,
                                   new Integer(m_sBuf.toString()).intValue());
          }

          //Reset our flag
          m_bIsIntData = false;

        }
        else if (m_bIsCharData) {
          if (m_bWantsTreeChar) {

            m_oData.AddTreeCharData(m_iSpecies, m_iType, m_iDataMemberCode,
                                    m_sBuf.toString());
          }

          //Reset our flag
          m_bIsCharData = false;

        }
        else if (m_bIsBoolData) {
          if (m_bWantsTreeBool) {

            m_oData.AddTreeBoolData(m_iSpecies, m_iType, m_iDataMemberCode,
                                    new Boolean(m_sBuf.toString()).booleanValue());
          }

          //Reset our flag
          m_bIsBoolData = false;

        }
        else if (m_bIsFloatCode) {
          if (m_bWantsTreeFloat) {

            if (m_iSpecies > -1 && m_iType > -1) {

              int iCode = new Integer(m_sBuf.toString()).intValue();
              m_oData.AddTreeFloatDataMemberCode(m_iSpecies, m_iType,
                                                 m_sDataMemberLabel,
                                                 iCode);
            }
          }

          //Reset our flag
          m_bIsFloatCode = false;

        }
        else if (m_bIsIntCode) {
          if (m_bWantsTreeInt) {

            if (m_iSpecies > -1 && m_iType > -1) {

              int iCode = new Integer(m_sBuf.toString()).intValue();
              m_oData.AddTreeIntDataMemberCode(m_iSpecies, m_iType,
                                               m_sDataMemberLabel,
                                               iCode);
            }
          }

          //Reset our flag
          m_bIsIntCode = false;

        }
        else if (m_bIsCharCode) {
          if (m_bWantsTreeChar) {

            if (m_iSpecies > -1 && m_iType > -1) {

              int iCode = new Integer(m_sBuf.toString()).intValue();
              m_oData.AddTreeCharDataMemberCode(m_iSpecies, m_iType,
                                                m_sDataMemberLabel,
                                                iCode);
            }
          }

          //Reset our flag
          m_bIsCharCode = false;

        }
        else if (m_bIsBoolCode) {
          if (m_bWantsTreeBool) {

            if (m_iSpecies > -1 && m_iType > -1) {

              int iCode = new Integer(m_sBuf.toString()).intValue();
              m_oData.AddTreeBoolDataMemberCode(m_iSpecies, m_iType,
                                                m_sDataMemberLabel,
                                                iCode);
            }
          }

          //Reset our flag
          m_bIsBoolCode = false;

        }
    }
    else {

      //*******************************************************
       //This is grid data
       //*******************************************************

        //Check our flags to see what we want to do
        if (m_bIsFloatData) {
          if (m_bWantsGridFloat) {

            m_oData.AddGridFloatData(m_sGridName, m_iX, m_iY, m_iDataMemberCode,
                                     new Float(m_sBuf.toString()).floatValue());
          }

          //Reset our flag
          m_bIsFloatData = false;

        }
        else if (m_bIsIntData) {
          if (m_bWantsGridInt) {

            m_oData.AddGridIntData(m_sGridName, m_iX, m_iY, m_iDataMemberCode,
                                   new Integer(m_sBuf.toString()).intValue());
          }

          //Reset our flag
          m_bIsIntData = false;

        }
        else if (m_bIsCharData) {
          if (m_bWantsGridChar) {

            m_oData.AddGridCharData(m_sGridName, m_iX, m_iY, m_iDataMemberCode,
                                    m_sBuf.toString());
          }

          //Reset our flag
          m_bIsCharData = false;

        }
        else if (m_bIsBoolData) {
          if (m_bWantsGridBool) {

            m_oData.AddGridBoolData(m_sGridName, m_iX, m_iY, m_iDataMemberCode,
                                    new Boolean(m_sBuf.toString()).booleanValue());

          }
          //Reset our flag
          m_bIsBoolData = false;

        }
        else if (m_bIsFloatCode) {
          if (m_bWantsGridFloat && m_bIsPackage == false) {

            int iCode = new Integer(m_sBuf.toString()).intValue();
            m_oData.AddGridFloatDataMemberCode(m_sGridName,
                                               m_sDataMemberLabel,
                                               iCode);
          }

          //Reset our flag
          m_bIsFloatCode = false;

        }
        else if (m_bIsIntCode) {
          if (m_bWantsGridInt && m_bIsPackage == false) {

            int iCode = new Integer(m_sBuf.toString()).intValue();
            m_oData.AddGridIntDataMemberCode(m_sGridName,
                                             m_sDataMemberLabel,
                                             iCode);
          }

          //Reset our flag
          m_bIsIntCode = false;

        }
        else if (m_bIsCharCode) {
          if (m_bWantsGridChar && m_bIsPackage == false) {

            int iCode = new Integer(m_sBuf.toString()).intValue();
            m_oData.AddGridCharDataMemberCode(m_sGridName,
                                              m_sDataMemberLabel,
                                              iCode);
          }

          //Reset our flag
          m_bIsCharCode = false;

        }
        else if (m_bIsBoolCode) {
          if (m_bWantsGridBool && m_bIsPackage == false) {

            int iCode = new Integer(m_sBuf.toString()).intValue();
            m_oData.AddGridBoolDataMemberCode(m_sGridName,
                                              m_sDataMemberLabel,
                                              iCode);
          }

          //Reset our flag
          m_bIsBoolCode = false;

        }
        else if (m_bIsGridXCellLength) {

          float fValue = new Float(m_sBuf.toString()).floatValue();
          m_oData.AddGridXCellLength(m_sGridName, fValue);
          m_bIsGridXCellLength = false;

        }
        else if (m_bIsGridYCellLength) {

          float fValue = new Float(m_sBuf.toString()).floatValue();
          m_oData.AddGridYCellLength(m_sGridName, fValue);
          m_bIsGridYCellLength = false;
        }
      //If this is the end of a set of package values, reset the flag
      //LEM 03-25-05
      if (sQName.startsWith("ma_package")) {
        m_bIsPackage = false;
      }
    }

  }

  /**
   * Reads character data from the XML file and appends it to the buffer.  The
   * parser can call this multiple times per tag.
   * @param ch The characters from the XML document.
   * @param start - The start position in the array.
   * @param length - The number of characters to read from the array.
   * @throws SAXException if any of the described cases above is true.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void characters(char[] ch,
                         int start,
                         int length) throws SAXException {

    m_sBuf.append(ch, start, length);

  }
}
