package javawrapper;

/**
 * An object of this class represents a data member from a grid or tree.  Not
 * all data members need to be set - just whatever is needed by whoever is
 * using this class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class DataMember
    implements Cloneable {

  public static final int INTEGER = 0;
  public static final int FLOAT = 1;
  public static final int CHAR = 2;
  public static final int BOOLEAN = 3;

  /**The display name is that which will be shown to users*/
  private String m_sDisplayName,
      /**The code name is that which will be written to detailed output - what
             the C++ code will recognize*/
      m_sCodeName;

  private int m_iType,
      /**<Data member type - one of the four static finals above*/
      m_iCode; /**<Data member code in detailed output files*/

  /**
   * Constructor
   * @param sDisplayName The name which will be displayed to the user
   * @param sCodeName The name of the data member to be passed to the core
   * model code.
   * @param iType The data member type - options are INTEGER, FLOAT, CHAR, and
   * BOOLEAN
   * @throws ModelException If iType is not recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember(String sDisplayName, String sCodeName, int iType) throws
      ModelException {

    if (iType < INTEGER || iType > BOOLEAN) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_ARGUMENT;
      oErr.sMessage = "A bad data member type has been passed - " + iType;
      oErr.sCallingFunction = "JAVA";
      throw (oErr);
    }

    m_iType = iType;
    m_sCodeName = sCodeName;
    m_sDisplayName = sDisplayName;
    m_iCode = -1;
  }

  /**
   * Overridden from Object.
   * @return Display name string
   */
  public String toString() {
    return m_sDisplayName;
  }

  /**
   * Constructor
   * @param sLabel The string to be used for both code name and display name.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember(String sLabel) {
    m_iCode = -1;
    m_iType = INTEGER;
    m_sCodeName = sLabel;
    m_sDisplayName = sLabel;
  }

  /**
   * Gets the display name of this data member.
   * @return The display name.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetDisplayName() {
    return m_sDisplayName;
  }

  /**
   * Gets the code name of this data member.
   * @return The code name.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetCodeName() {
    return m_sCodeName;
  }

  /**
   * Gets the type of this data member.
   * @return Data member type.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetType() {
    return m_iType;
  }

  /**
   * Get the data member numerical code.
   * @return The code number.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetCode() {
    return m_iCode;
  }

  /**
   * Sets the data member numerical code.
   * @param iCode The code number.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetCode(int iCode) {
    m_iCode = iCode;
  }

  /**
   * Creates a deep clone of this object.
   * @return Clone.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Object clone() {
    try {
      DataMember oClone = new DataMember(m_sDisplayName, m_sCodeName, m_iType);
      oClone.SetCode(m_iCode);
      return oClone;
    }
    catch (ModelException oErr) {
      return null; //Do nothing - assume that this was a valid object
    }
  }
}
