package javawrapper;


/**
 * Plot object.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class Plot
    extends WorkerBase {

  public static final float CELL_LENGTH = (float)8.0;

  protected ModelString
      /**Plot title or comment*/
      m_sPlotTitle = new ModelString("", "Plot title", "plot_title");

  /**Length of plot in X direction.  X is EW.*/
  protected ModelFloat m_fPlotLenX = new ModelFloat(0,
      "Plot Length in the X (E-W) Direction, in meters", "plot_lenX");

  /**Length of plot in Y direction.  Y is NS.*/
  protected ModelFloat m_fPlotLenY = new ModelFloat(0,
      "Plot Length in the Y (N-S) Direction, in meters", "plot_lenY");

  /**Latitude of plot in decimal degrees*/
  protected ModelFloat m_fLatitude = new ModelFloat(0,
      "Plot Latitude, in decimal degrees", "plot_latitude");

  /**Number of years per timestep - default to 5, the old value*/
  protected ModelFloat m_fNumberOfYearsPerTimestep = new ModelFloat(5,
      "Number of years per timestep", "yearsPerTimestep");

  /**Number of timesteps for the run*/
  protected ModelInt m_iNumTimesteps = new ModelInt(0, "Number of Timesteps",
      "timesteps");

  /**Run's random seed*/
  protected ModelInt m_iRandomSeed = new ModelInt(0, "Random Seed",
                                                  "randomSeed");

  /**Current timestep, if a detailed output timestep file has
   been fed in.  Not required.*/
  protected ModelInt m_iCurrentTimestep = new ModelInt(0, "Current Timestep",
      "rt_timestep");

  protected int m_iNumOldFileXCells,
      /**<Number of X cells - for reading in old files only*/
      m_iNumOldFileYCells; /**<Number of Y cells - for reading in old files only*/
  protected float m_iNumOldFileXGridLength,
      /**<Length of X cells - for reading in old files only*/
      m_iNumOldFileYGridLength; /**<Length of Y cells - for reading in old files only*/

  /**
   * Default constructor initializes all values to the equivalent of 0.
   * @param oManager GUIManager object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Plot(GUIManager oManager) {
    super(oManager, "Plot", "plot");

    m_sHelpID = "data.plot";

    mp_oAllData.add(m_iNumTimesteps);
    mp_oAllData.add(m_iCurrentTimestep);
    mp_oAllData.add(m_iRandomSeed);
    mp_oAllData.add(m_fNumberOfYearsPerTimestep);
    mp_oAllData.add(m_fPlotLenX);
    mp_oAllData.add(m_fPlotLenY);
    mp_oAllData.add(m_fLatitude);
    mp_oAllData.add(m_sPlotTitle);

    m_iNumOldFileXCells = 0;
    m_iNumOldFileYCells = 0;
    m_iNumOldFileXGridLength = 0;
    m_iNumOldFileYGridLength = 0;
  }

  /**Does nothing
   * @param oPop Ignored*/
  public void DoSetup(TreePopulation oPop){;}

  /**
   * Sets the number of old grid cells in the X direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be number
   * of cells in the Y direction).
   * @param i Number of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetOldFileXGridCells(int i) {
    m_iNumOldFileXCells = i;
  }

  /**
   * Sets the number of old grid cells in the Y direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be number
   * of cells in the X direction).
   * @param i Number of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetOldFileYGridCells(int i) {
    m_iNumOldFileYCells = i;
  }

  /**
   * Sets the number of years per timestep.
   * @param fYears Number of years per timestep.
   * @throws ModelException if the number of years per timestep is not greater
   * than 0.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetYearsPerTimestep(float fYears) throws ModelException {
    ModelFloat fTest = new ModelFloat(fYears,
                                      m_fNumberOfYearsPerTimestep.GetDescriptor(),
                                      "");
    MakeSurePositive(fTest);
    m_fNumberOfYearsPerTimestep.SetValue(fYears);
  }

  /**
   * Gets the number of old grid cells in the X direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be number
   * of cells in the Y direction).
   * @return Number of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetOldFileXGridCells() {
    return m_iNumOldFileXCells;
  }

  /**
   * Gets the number of old grid cells in the Y direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be number
   * of cells in the X direction).
   * @return Number of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetOldFileYGridCells() {
    return m_iNumOldFileYCells;
  }

  /**
   * Sets the length of old grid cells in the X direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be length
   * of cells in the Y direction).
   * @param f Length of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetOldFileXGridCellLength(float f) {
    m_iNumOldFileXGridLength = f;
  }

  /**
   * Sets the length of old grid cells in the Y direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be length
   * of cells in the X direction).
   * @param f Length of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetOldFileYGridCellLength(float f) {
    m_iNumOldFileYGridLength = f;
  }

  /**
   * Gets the length of old grid cells in the X direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be length
   * of cells in the Y direction).
   * @return Length of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetOldFileXGridCellLength() {
    return m_iNumOldFileXGridLength;
  }

  /**
   * Gets the length of old grid cells in the Y direction IN THE NEW COORDINATE
   * SYSTEM (i.e. in an old tab-delimited parameter file, this would be length
   * of cells in the X direction).
   * @return Length of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetOldFileYGridCellLength() {
    return m_iNumOldFileYGridLength;
  }

  /**
   * Gets the number of years per timestep.
   * @return Number of years per timestep.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetNumberOfYearsPerTimestep() {
    return m_fNumberOfYearsPerTimestep.GetValue();
  }

  /**
   * Sets the plot title.
   * @param sTitle Title of the plot.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPlotTitle(String sTitle) {
    m_sPlotTitle.SetValue(sTitle);
  }

  /**
   * Sets the number of timesteps for a run.
   * @param i The number of timesteps.
   * @throws ModelException when the number of timesteps is less than or equal
   * to zero.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetNumberOfTimesteps(int i) throws ModelException {
    ModelInt iTest = new ModelInt(i, m_iNumTimesteps.GetDescriptor(), "");
    MakeSurePositive(iTest);
    m_iNumTimesteps.SetValue(i);
  }

  /**
   * Gets the number of timesteps for a run.
   * @return The number of timesteps.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>June 1, 2004: Created (LEM)
   */
  public int GetNumberOfTimesteps() {
    return m_iNumTimesteps.GetValue();
  }

  /**
   * Sets the random seed for the run.  A value of 0 means a new random seed each time a parameter file is run.
   * @param i The random seed value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetRandomSeed(int i) {
    m_iRandomSeed.SetValue(i);
  }

  /**
   * Sets the plot length in the X (East-West) direction.
   * @param f Length of the plot in the X direction.
   * @throws ModelException if the passed length is less than or equal to 0.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPlotXLength(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPlotLenX.GetDescriptor(), "");
    MakeSurePositive(fTest);
    m_fPlotLenX.SetValue(f);
  }

  /**
   * Gets the current plot X (East-West) length.
   * @return Current plot X length.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetPlotXLength() {
    return m_fPlotLenX.GetValue();
  }

  /**
   * Sets the plot length in the Y (North-South) direction.
   * @param f Length of the plot in the Y direction.
   * @throws ModelException if the passed length is less than or equal to 0.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetPlotYLength(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPlotLenY.GetDescriptor(), "");
    MakeSurePositive(fTest);
    m_fPlotLenY.SetValue(f);
  }

  /**
   * Gets the current plot Y (North-South) length.
   * @return Current plot Y length.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetPlotYLength() {
    return m_fPlotLenY.GetValue();
  }

  /**
   * Sets the latitude of the plot.
   * @param f Latitude in decimal degrees.
   * @throws ModelException if latitude is not between -90 and 90.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetLatitude(float f) throws ModelException {
    if (f > 90 || f < -90) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = m_fLatitude.GetDescriptor() +
          " must be between -90 and 90.";
      throw (oErr);
    }
    m_fLatitude.SetValue(f);
  }

  /**
   * Validates the data prior to running the model.
   * @throws ModelException if any of the following are not greater than zero:
   * <ul>
   * <li> m_fPlotLenX
   * <li> m_fPlotLenY
   * <li> m_iNumTimesteps
   * <li> m_fNumberOfYearsPerTimestep
   * </ul>
   * or if m_fLatitude is not between -90 and +90.
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    MakeSurePositive(m_fPlotLenX);
    MakeSurePositive(m_fPlotLenY);
    MakeSurePositive(m_iNumTimesteps);
    MakeSurePositive(m_fNumberOfYearsPerTimestep);
    if (m_fLatitude.GetValue() > 90 || m_fLatitude.GetValue() < -90) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = m_fLatitude.GetDescriptor() +
          " must be between -90 and 90.";
      throw (oErr);
    }
  }
}
