package javawrapper;

import junit.framework.*;
import java.util.Vector;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the GrowthBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestGrowthBehaviors
    extends TestCase {
  private TreePopulation m_oPop = null;

  public TestGrowthBehaviors(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
    try {
      //Create our tree population
      m_oPop = new TreePopulation(null);
      m_oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});
    }
    catch (ModelException oErr) {
      throw new Exception(oErr.sMessage);
    }
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * This test makes sure that lambdas are reset correctly if species are
   * changed.
   */
  public void testDoSetup() {
    try {
      ModelVector p_oData;
      int i;

      //First - set up the way it is
      GUIManager oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      oGrowth.DoSetup(oManager.GetTreePopulation());

      //Set some lambda values
      Float[] p_fFloat = new Float[] {new Float(0.01), new Float(0.02)};
    Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
    for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
      ModelData oData = (ModelData) oBeh.GetRequiredData(i);
      String sDescriptor = oData.GetDescriptor().toLowerCase();
      if (sDescriptor.indexOf("lambda") > -1) {
        p_oData = (ModelVector) oBeh.GetRequiredData(i);
        oGrowth.SetVectorValues(p_oData, p_fFloat);
        break;
      }
    }

      p_fFloat[0] = new Float(0.11);
      p_fFloat[1] = new Float(0.12);
      for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
      ModelData oData = (ModelData) oBeh.GetRequiredData(i);
      String sDescriptor = oData.GetDescriptor().toLowerCase();
      if (sDescriptor.indexOf("lambda") > -1) {
        p_oData = (ModelVector) oBeh.GetRequiredData(i+1);
        oGrowth.SetVectorValues(p_oData, p_fFloat);
        break;
      }
    }

      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      boolean bFoundNCIACRU = false,
          bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      //************************
       // Now remove a species
       //************************
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU"});
      //Make sure there are ACRU lambda neighbors only in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
            //Make sure it has the values we set for it
            p_oData = (ModelVector) oData;
            assertEquals(0.01,((Float)p_oData.GetValue().elementAt(0)).doubleValue(), 0.001);
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in all data when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in behavior when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      //************************
       //Reset our species back to normal
       //************************
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }
  }

  public void testFormatDataForDisplay() {
    try {
      GrowthBehaviors oGrowth = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no growth behaviors are enabled.
      oGrowth = new GrowthBehaviors(null);
      assertEquals(null, oGrowth.FormatDataForDisplay(m_oPop));

      //Case:  only absolute radial growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("absradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only absolute basal area growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("absba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Area Growth in sq. cm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only absolute unlimited growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("absunlim");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only relative radial growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("relradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only relative basal area growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("relba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only relative unlimited growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("relunlim");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only constant radial growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("constradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only constant basal area growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("constba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only nci growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "ACRU NCI Lambda Neighbors",
          "ACSA NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only logistic growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("logistic growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only simple linear growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("linear");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Diam Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only linear bi-level growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("linear bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("size dependent logistic growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only lognormal growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only shaded linear growth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("shaded linear");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only double-resource relative gorwth, auto-diam, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("double resource");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Double resource - Influence of Resource (C)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only absolute radial growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("absradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only absolute basal area growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("absba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Area Growth in sq. cm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only absolute unlimited growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("absunlim diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only relative radial growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("relradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only relative basal area growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("relba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only relative unlimited growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("relunlim diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only constant radial growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("constradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only constant basal area growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("constba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only nci growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("nci diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "ACRU NCI Lambda Neighbors",
          "ACSA NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only logistic growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("logistic growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only linear bi-level growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("linear bi-level diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only Simple Linear growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("linear diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Diam Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey(
          "size dependent logistic growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only lognormal growth, Diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only Shaded Linear growth, Diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("shaded linear diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only double-resource relative growth, diam only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("double resource diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Double resource - Influence of Resource (C)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only height incrementer is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("height incrementer");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      assertEquals(null, oGrowth.FormatDataForDisplay(m_oPop));

      //Case:  only logistic growth, height only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("logistic growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
          "Logistic - Height Shape Param 1 (b)",
          "Logistic - Height Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only Simple Linear growth, height only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("linear height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Simple Linear - Height Intercept in cm/yr (a)",
          "Simple Linear - Height Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, height only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey(
          "size dependent logistic growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Height Intercept (a)",
          "Size Dep. Logistic - Height Slope (b)",
          "Size Dep. Logistic - Height Shape Param 1 (c)",
          "Size Dep. Logistic - Height Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only lognormal growth, height only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
          "Lognormal - Height Shape Parameter (b)",
          "Lognormal - Height Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only Shaded Linear growth, height only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("shaded linear height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Height Intercept in cm/yr (a)",
          "Shaded Linear - Height Slope (b)",
          "Shaded Linear - Height Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only Shaded Linear growth, height only, is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Lognormal Bi-Level - X0 for Low-Light Growth",
          "Lognormal Bi-Level - Xb for Low-Light Growth",
          "Lognormal Bi-Level - X0 for High-Light Growth",
          "Lognormal Bi-Level - Xb for High-Light Growth",
          "Lognormal Bi-Level - Max Growth in High Light (m)",
          "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Lognormal Bi-Level - Max Growth in Low Light (m)"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only PR semi-stochastic diameter growth is enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      oBeh = oGrowth.GetBehaviorByKey("pr semi-stochastic diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "PR - Height Threshold for Stochastic Growth (m)",
          "PR - \"a\" Parameter for Deterministic Growth",
          "PR - \"b\" Parameter for Deterministic Growth",
          "PR - Mean DBH (cm) for Stochastic Growth",
          "PR - DBH Standard Deviation for Stochastic Growth"};
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);


      //Case:  all are enabled.
      oGrowth = null;
      oGrowth = new GrowthBehaviors(null);
      oGrowth.DoSetup(m_oPop);
      int i;
      for (i = 0; i < oGrowth.mp_oChildBehaviors.length; i++) {
        oGrowth.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oGrowth.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new SpeciesTypeCombo(
            0, 3, m_oPop));
      }
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "ACRU NCI Lambda Neighbors",
          "ACSA NCI Lambda Neighbors",
          "Slope of Growth Response (S)",
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Adult Constant Area Growth in sq. cm/yr",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor",
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Height Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
          "Logistic - Height Shape Param 2 (c)",
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Height Intercept in cm/yr (a)",
          "Simple Linear - Diam Slope (b)",
          "Simple Linear - Height Slope (b)",
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Height Intercept in cm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Height Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)",
          "Shaded Linear - Height Shade Exponent (c)",
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Height Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Height Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Height Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)",
          "Size Dep. Logistic - Height Shape Param 2 (d)",
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Height Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)",
          "Lognormal - Height Effect of Shade (c)",
          "Double resource - Influence of Resource (C)",
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)",
          "Lognormal Bi-Level - X0 for Low-Light Growth",
          "Lognormal Bi-Level - Xb for Low-Light Growth",
          "Lognormal Bi-Level - X0 for High-Light Growth",
          "Lognormal Bi-Level - Xb for High-Light Growth",
          "Lognormal Bi-Level - Max Growth in High Light (m)",
          "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Lognormal Bi-Level - Max Growth in Low Light (m)",
          "PR - Height Threshold for Stochastic Growth (m)",
          "PR - \"a\" Parameter for Deterministic Growth",
          "PR - \"b\" Parameter for Deterministic Growth",
          "PR - Mean DBH (cm) for Stochastic Growth",
          "PR - DBH Standard Deviation for Stochastic Growth"
      };
      TestTable(oGrowth.FormatDataForDisplay(m_oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for growth.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }
  }

  /**
   * Makes sure that nci lambda values are correctly read from parameter
   * file.
   */
  public void testReadParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.6640108, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.71, 0.01);
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.00442797, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.12, 0.01);

      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

  }

  /**
   * Tests ValidateData().
   */
  public void testValidateData() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 2
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 10
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile10();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 10.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 11
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile11();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 11.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 12
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile12();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 12.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 13
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile13();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 13.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 14
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile14();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 14.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 15
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile15();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 15.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 16
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile16();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 16.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 17
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile17();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 17.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 18
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile18();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 18.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Light threshold not between 0 and 100 for linear bi-level growth
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("linear bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Light threshold not between 0 and 100 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light X0 = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelLoLiteX0.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelLoLiteX0.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad X0 value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light X0 = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelHiLiteX0.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelHiLiteX0.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad X0 value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light Xb = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelLoLiteXb.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelLoLiteXb.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad Xb value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light Xb = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelHiLiteXb.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelHiLiteXb.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad Xb value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light max growth < 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelHiLiteMaxGrwth.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelHiLiteMaxGrwth.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad max growth value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light max growth < 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, m_oPop));
      oGrowth.mp_fLogBiLevelLoLiteMaxGrwth.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelLoLiteMaxGrwth.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad max growth value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }






    System.out.println("Growth ValidateData testing succeeded.");
  }

  /**
   * Tests a table set against a list of expected members in the table.  It
   * does not care how many tables are defined in the vector.  It will
   * throw a fail if there are members missing or members that are not
   * expected.  Species are not checked.  Blanks are skipped.
   * @param p_oTablesToTest Vector of tables to test.
   * @param p_sExpected Expected member list.
   */
  private void TestTable(Vector p_oTablesToTest, String[] p_sExpected) {
    Object[][] p_oData;
    Vector p_oOneTable;
    String sMemberName;
    int[] p_iFoundTimes = new int[p_sExpected.length]; //holds the number of
    //times each member was found
    int iNumberOfTables = p_oTablesToTest.size(),
        i, j, k;
    boolean bFound;

    //Initialize the number of times found array to all 0s
    for (i = 0; i < p_iFoundTimes.length; i++) {
      p_iFoundTimes[i] = 0;
    }

    //Analyze each table
    for (i = 0; i < iNumberOfTables; i++) {
      p_oOneTable = (Vector) p_oTablesToTest.elementAt(i);

      //The second element in the vector is the data
      p_oData = (Object[][]) p_oOneTable.elementAt(1);

      //Go through each table row
      for (j = 0; j < p_oData.length; j++) {

        sMemberName = (String) p_oData[j][0];
        bFound = false;

        if (!sMemberName.equals(" ")) {

          //Does it match a member of the expected list?
          for (k = 0; k < p_sExpected.length; k++) {

            if (sMemberName.equals(p_sExpected[k])) {

              bFound = true;
              p_iFoundTimes[k]++;

            }
          }
        }

        //Did we find our member in the expected list?
        if (!bFound) {
          fail(sMemberName + " unexpectedly found in table.");
        }
      }
    }

    //Now look at the number of times we found each expected member - if
    //not 1 for each, fail
    for (i = 0; i < p_iFoundTimes.length; i++) {
      assertEquals("Found " + p_sExpected[i] + " unexpected number of times.",
                   1, p_iFoundTimes[i]);
    }
  }

  /**
   * Writes a valid PR NCI growth file, Diam with auto height.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid NCI growth file, Diam only.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile1End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile1End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile2End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile2End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile3End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile3End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile4End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile4End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile5End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile10() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile5End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile11() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile6End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile12() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile6End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile13() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile7End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile14() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile7End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile15() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile8End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile16() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile8End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile17() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile9End(oOut);
    oOut.close();
    return sFileName;
  }

  protected String WriteXMLErrorFile18() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile9End(oOut);
    oOut.close();
    return sFileName;
  }

  private void WriteValidateFilePart1(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  private void WriteBehaviorListDiamAuto(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  private void WriteBehaviorListDiamOnly(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth diam only</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  private void WriteNCIMaxCrowdingRadius(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Species_2\">10</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
  }

  private void WriteNCIMinNeighborDBH(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciMinNeighborDBH>");
    oOut.write("<gr_nmndVal species=\"Species_1\">10</gr_nmndVal>");
    oOut.write("<gr_nmndVal species=\"Species_2\">10</gr_nmndVal>");
    oOut.write("</gr_nciMinNeighborDBH>");
  }

  private void WriteNCINeighborDBHEffect(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciAlpha>");
    oOut.write("<gr_naVal species=\"Species_1\">2.17031683</gr_naVal>");
    oOut.write("<gr_naVal species=\"Species_2\">2.81</gr_naVal>");
    oOut.write("</gr_nciAlpha>");
  }

  private void WriteNCINeighborDistEffect(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciBeta>");
    oOut.write("<gr_nbVal species=\"Species_1\">0.699942</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Species_2\">0.5</gr_nbVal>");
    oOut.write("</gr_nciBeta>");
  }

  private void WriteNCIMaxPotentialGrowth(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Species_1\">10.5258749</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Species_2\">15.31</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
  }

  private void WriteNCISizeSensNCI(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciSizeSensNCI>");
    oOut.write("<gr_nssnVal species=\"Species_1\">0.43</gr_nssnVal>");
    oOut.write("<gr_nssnVal species=\"Species_2\">0.36</gr_nssnVal>");
    oOut.write("</gr_nciSizeSensNCI>");
  }

  private void WriteNCISizeEffectMode(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciSizeEffectMode>");
    oOut.write("<gr_nsemVal species=\"Species_1\">34.24638994</gr_nsemVal>");
    oOut.write("<gr_nsemVal species=\"Species_2\">122.23</gr_nsemVal>");
    oOut.write("</gr_nciSizeEffectMode>");
  }

  private void WriteNCISizeEffectVar(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciSizeEffectVariance>");
    oOut.write("<gr_nsevVal species=\"Species_1\">5</gr_nsevVal>");
    oOut.write("<gr_nsevVal species=\"Species_2\">2.36</gr_nsevVal>");
    oOut.write("</gr_nciSizeEffectVariance>");
  }

  private void WriteNCISlope(FileWriter oOut) throws java.io.IOException {
    oOut.write("<gr_nciCrowdingSlope>");
    oOut.write("<gr_ncslVal species=\"Species_1\">1.98</gr_ncslVal>");
    oOut.write("<gr_ncslVal species=\"Species_2\">4.57</gr_ncslVal>");
    oOut.write("</gr_nciCrowdingSlope>");
  }

  private void WriteNCISteepness(FileWriter oOut) throws java.io.IOException {
    oOut.write("<gr_nciCrowdingSteepness>");
    oOut.write("<gr_ncstVal species=\"Species_1\">1.63</gr_ncstVal>");
    oOut.write("<gr_ncstVal species=\"Species_2\">1.26</gr_ncstVal>");
    oOut.write("</gr_nciCrowdingSteepness>");
  }

  private void WriteNCIStormEffMedDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciStormEffMedDmg>");
    oOut.write("<gr_nsemdVal species=\"Species_1\">0.89</gr_nsemdVal>");
    oOut.write("<gr_nsemdVal species=\"Species_2\">0.9</gr_nsemdVal>");
    oOut.write("</gr_nciStormEffMedDmg>");
  }

  private void WriteNCIStormEffFullDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciStormEffFullDmg>");
    oOut.write("<gr_nsefdVal species=\"Species_1\">0.42</gr_nsefdVal>");
    oOut.write("<gr_nsefdVal species=\"Species_2\">0.34</gr_nsefdVal>");
    oOut.write("</gr_nciStormEffFullDmg>");
  }

  private void WriteNCINeighStormEffMedDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciNeighStormEffMedDmg>");
    oOut.write("<gr_nnsemdVal species=\"Species_1\">0.89</gr_nnsemdVal>");
    oOut.write("<gr_nnsemdVal species=\"Species_2\">0.9</gr_nnsemdVal>");
    oOut.write("</gr_nciNeighStormEffMedDmg>");
  }

  private void WriteNCINeighStormEffFullDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciNeighStormEffFullDmg>");
    oOut.write("<gr_nnsefdVal species=\"Species_1\">0.42</gr_nnsefdVal>");
    oOut.write("<gr_nnsefdVal species=\"Species_2\">0.34</gr_nnsefdVal>");
    oOut.write("</gr_nciNeighStormEffFullDmg>");
  }

  private void WriteNCIShadingCoefficient(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciShadingCoefficient>");
    oOut.write("<gr_nscVal species=\"Species_1\">0.42</gr_nscVal>");
    oOut.write("<gr_nscVal species=\"Species_2\">0.34</gr_nscVal>");
    oOut.write("</gr_nciShadingCoefficient>");
  }

  private void WriteNCIShadingExponent(FileWriter oOut) throws java.io.
        IOException {
      oOut.write("<gr_nciShadingExponent>");
      oOut.write("<gr_nseVal species=\"Species_1\">0.42</gr_nseVal>");
      oOut.write("<gr_nseVal species=\"Species_2\">0.34</gr_nseVal>");
      oOut.write("</gr_nciShadingExponent>");
    }

  private void WriteNCILambdas(FileWriter oOut) throws java.io.IOException {
    oOut.write("<gr_nciSpecies_1NeighborLambda>");
    oOut.write("<gr_nlVal species=\"Species_1\">0.6640108</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Species_2\">0.71</gr_nlVal>");
    oOut.write("</gr_nciSpecies_1NeighborLambda>");
    oOut.write("<gr_nciSpecies_2NeighborLambda>");
    oOut.write("<gr_nlVal species=\"Species_1\">0.00442797</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Species_2\">0.12</gr_nlVal>");
    oOut.write("</gr_nciSpecies_2NeighborLambda>");

  }

  private void WriteValidFileEnd(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where max radius of neighbor Effects is < 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile1End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Species_2\">-10</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCIShadingExponent(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where max growth for a species is < 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile2End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Species_1\">15.31</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Species_2\">-15.31</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where Xb for a species = 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile3End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    oOut.write("<gr_nciSizeEffectVariance>");
    oOut.write("<gr_nsevVal species=\"Species_1\">0</gr_nsevVal>");
    oOut.write("<gr_nsevVal species=\"Species_2\">5</gr_nsevVal>");
    oOut.write("</gr_nciSizeEffectVariance>");
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where X0 for a species = 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile4End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    oOut.write("<gr_nciSizeEffectMode>");
    oOut.write("<gr_nsemVal species=\"Species_1\">0</gr_nsemVal>");
    oOut.write("<gr_nsemVal species=\"Species_2\">34.2</gr_nsemVal>");
    oOut.write("</gr_nciSizeEffectMode>");
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where med. damage eta for a species is not a
   * proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile5End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    oOut.write("<gr_nciNeighStormEffMedDmg>");
    oOut.write("<gr_nnsemdVal species=\"Species_1\">-0.33</gr_nnsemdVal>");
    oOut.write("<gr_nnsemdVal species=\"Species_2\">0.33</gr_nnsemdVal>");
    oOut.write("</gr_nciNeighStormEffMedDmg>");
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where full damage eta for a species is not a
   * proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile6End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    oOut.write("<gr_nciNeighStormEffFullDmg>");
    oOut.write("<gr_nnsefdVal species=\"Species_1\">-0.54</gr_nnsefdVal>");
    oOut.write("<gr_nnsefdVal species=\"Species_2\">0.54</gr_nnsefdVal>");
    oOut.write("</gr_nciNeighStormEffFullDmg>");
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where med. damage storm Effect for a species is
   * not a proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile7End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    oOut.write("<gr_nciStormEffMedDmg>");
    oOut.write("<gr_nsemdVal species=\"Species_1\">0.9</gr_nsemdVal>");
    oOut.write("<gr_nsemdVal species=\"Species_2\">1.9</gr_nsemdVal>");
    oOut.write("</gr_nciStormEffMedDmg>");
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where full damage storm Effect for a species is
   * not a proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile8End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    oOut.write("<gr_nciStormEffFullDmg>");
    oOut.write("<gr_nsefdVal species=\"Species_1\">-0.42</gr_nsefdVal>");
    oOut.write("<gr_nsefdVal species=\"Species_2\">0.42</gr_nsefdVal>");
    oOut.write("</gr_nciStormEffFullDmg>");
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where minimum neighbor DBH is less than 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile9End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    oOut.write("<gr_nciMinNeighborDBH>");
    oOut.write("<gr_nmndVal species=\"Species_1\">10</gr_nmndVal>");
    oOut.write("<gr_nmndVal species=\"Species_2\">-10</gr_nmndVal>");
    oOut.write("</gr_nciMinNeighborDBH>");
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }
}
