package javawrapper;

import junit.framework.*;

/**
 * Tests the PlantingData class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestPlantingData
    extends TestCase {
  private PlantingData m_oPlantingData = null;

  protected void setUp() throws Exception {
    super.setUp();
    m_oPlantingData = new PlantingData();
  }

  protected void tearDown() throws Exception {
    m_oPlantingData = null;
    super.tearDown();
  }

  public void testAddGetAbundance() {

    //Normal processing
    int iSpecies = 0;
    float fAbundance = 0;

    try {

      //First abundance added
      m_oPlantingData.AddAbundance(iSpecies, fAbundance);
      assertEquals(fAbundance, m_oPlantingData.GetAbundance(iSpecies), 0);

      //Second abundance added
      iSpecies = 4;
      fAbundance = 14;
      m_oPlantingData.AddAbundance(iSpecies, fAbundance);
      assertEquals(fAbundance, m_oPlantingData.GetAbundance(iSpecies), 0);

      //Replace the abundance for a species
      iSpecies = 4;
      fAbundance = (float) 0.4;
      m_oPlantingData.AddAbundance(iSpecies, fAbundance);
      assertEquals(fAbundance, m_oPlantingData.GetAbundance(iSpecies), 0);

    }
    catch (ModelException oErr) {
      fail("Normal processing for AddAbundance() failed with message " +
           oErr.sMessage);
    }

    //Exception processing - negative abundance
    try {
      iSpecies = 3;
      fAbundance = -5;
      m_oPlantingData.AddAbundance(iSpecies, fAbundance);
      fail("AddAbundance() did not catch negative abundance value of " +
           fAbundance + ".");
    }
    catch (ModelException oErr) {
      System.out.println("Exception processing succeeded for AddAbundance().");
    }

  }

  public void testCell() {
    Plot oPlot = null;
    //Normal processing
    try {
      oPlot = new Plot(null);
      oPlot.SetOldFileXGridCellLength(10);
      oPlot.SetPlotXLength(100);
      oPlot.SetOldFileYGridCellLength(10);
      oPlot.SetPlotYLength(100);

      assertEquals(m_oPlantingData.GetNumberOfCells(), 0);

      m_oPlantingData.AddCell(0, 0, oPlot);
      assertEquals(m_oPlantingData.GetNumberOfCells(), 1);
      assertEquals(0, m_oPlantingData.GetCell(0).GetX());
      assertEquals(0, m_oPlantingData.GetCell(0).GetY());

      m_oPlantingData.AddCell(9, 9, oPlot);
      assertEquals(m_oPlantingData.GetNumberOfCells(), 2);
      assertEquals(9, m_oPlantingData.GetCell(1).GetX());
      assertEquals(9, m_oPlantingData.GetCell(1).GetY());

      m_oPlantingData.AddCell(3, 4, oPlot);
      assertEquals(m_oPlantingData.GetNumberOfCells(), 3);
      assertEquals(3, m_oPlantingData.GetCell(2).GetX());
      assertEquals(4, m_oPlantingData.GetCell(2).GetY());

      //Duplicate cell
      m_oPlantingData.AddCell(9, 9, oPlot);
      assertEquals(m_oPlantingData.GetNumberOfCells(), 3);

      //New cell passed as cell
      Cell oCell = new Cell(2, 6, oPlot);
      m_oPlantingData.AddCell(oCell);
      assertEquals(4, m_oPlantingData.GetNumberOfCells());
      assertEquals(2, m_oPlantingData.GetCell(3).GetX());
      assertEquals(6, m_oPlantingData.GetCell(3).GetY());

      //Duplicate passed as cell
      oCell = new Cell(3, 4, oPlot);
      m_oPlantingData.AddCell(oCell);
      assertEquals(4, m_oPlantingData.GetNumberOfCells());

    }
    catch (ModelException oErr) {
      fail("Normal processing failed for cells with message " +
           oErr.sMessage);
    }

    System.out.println("Normal processing succeeded for cells.");

    //Exception processing
    try {
      //Bad cell index
      m_oPlantingData.GetCell(6);
      fail("GetCell did not reject bad cell index.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      //Bad cell index
      m_oPlantingData.GetCell( -6);
      fail("GetCell did not reject bad cell index.");
    }
    catch (ModelException oErr) {
      ;
    }
    System.out.println("Exception processing succeeded for cells.");

  }

  public void testAddGetSpecies() {
    int iSpecies, iActual;

    //No value has been set
    iSpecies = 0;
    iActual = m_oPlantingData.GetNumberOfSpecies();
    assertEquals(iActual, iSpecies);

    //No value has been set - attempting to get species out
    try {
      iSpecies = 1;
      iActual = m_oPlantingData.GetSpecies(iSpecies);
      fail("GetSpecies did not throw an error when there were no species.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Add some values
      iSpecies = 0;
      m_oPlantingData.AddSpecies(iSpecies);
      iActual = m_oPlantingData.GetNumberOfSpecies();
      assertEquals(iActual, 1);
      iActual = m_oPlantingData.GetSpecies(0);
      assertEquals(iActual, iSpecies);

      iSpecies = 2;
      m_oPlantingData.AddSpecies(iSpecies);
      iActual = m_oPlantingData.GetNumberOfSpecies();
      assertEquals(iActual, 2);
      iActual = m_oPlantingData.GetSpecies(1);
      assertEquals(iActual, iSpecies);

      //Adding duplicate
      m_oPlantingData.AddSpecies(iSpecies);
      iActual = m_oPlantingData.GetNumberOfSpecies();
      assertEquals(iActual, 2);
      iActual = m_oPlantingData.GetSpecies(1);
      assertEquals(iActual, iSpecies);

    }
    catch (ModelException oErr) {
      fail("Normal processing for AddSpecies, GetNumberOfSpecies, and GetSpecies failed with message " +
           oErr.sMessage);
    }

    //Exception processing
    try {
      iActual = m_oPlantingData.GetSpecies(3);
      fail(
          "GetSpecies did not throw an error when the species index was too large.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      iActual = m_oPlantingData.GetSpecies( -3);
      fail(
          "GetSpecies did not throw an error when the species index was negative.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "Normal processing succeeded for species getting and setting.");
    System.out.println(
        "Exception processing succeeded for species getting and setting.");

  }

  public void testGetSetPlantSpacing() {
    //No value has been set
    int iPlanting = -1;
    int iActualReturn = m_oPlantingData.GetPlantSpacing();
    assertEquals("return value", -1, iActualReturn);

    try {
      //Normal processing
      iPlanting = PlantingBehaviors.GRIDDED;
      m_oPlantingData.SetPlantSpacing(iPlanting);
      iActualReturn = m_oPlantingData.GetPlantSpacing();
      assertEquals(iPlanting, iActualReturn);

      iPlanting = PlantingBehaviors.RANDOM;
      m_oPlantingData.SetPlantSpacing(iPlanting);
      iActualReturn = m_oPlantingData.GetPlantSpacing();
      assertEquals(iPlanting, iActualReturn);

      System.out.println(
          "Normal processing succeeded for GetPlantSpacing() and SetPlantSpacing().");
    }
    catch (ModelException oErr) {
      fail("Normal processing for GetPlantSpacing() and SetPlantSpacing() " +
           " failed with message " + oErr.sMessage + ".");
    }

    //Exception processing
    try {
      iPlanting = -2;
      m_oPlantingData.SetPlantSpacing(iPlanting);
      fail("SetPlantSpacing() did not catch passed value of " + iPlanting + ".");
    }
    catch (ModelException oErr) {
      System.out.println(
          "Exception processing succeeded for GetPlantSpacing() and SetPlantSpacing().");
    }
  }

  public void testGetSetTimestep() {
    int iTimestep, iActual;

    try {
      iTimestep = 0;
      m_oPlantingData.SetTimestep(iTimestep);
      iActual = m_oPlantingData.GetTimestep();
      assertEquals(iActual, iTimestep);

      iTimestep = 10;
      m_oPlantingData.SetTimestep(iTimestep);
      iActual = m_oPlantingData.GetTimestep();
      assertEquals(iActual, iTimestep);

    }
    catch (ModelException oErr) {
      fail("Normal processing for SetTimestep failed with message " +
           oErr.sMessage);
    }

    try {
      iTimestep = -3;
      m_oPlantingData.SetTimestep(iTimestep);
      fail("SetTimestep did not catch a timestep value of " +
           String.valueOf(iTimestep));
    }
    catch (ModelException oErr) {
      System.out.println("Exception processing succeeded for SetTimestep.");
    }
  }

  public void testGetSetSpacingDistance() {
    float fSpace = 0;
    try {

      //No value set
      assertEquals(0, m_oPlantingData.GetSpacingDistance(), 0);
      m_oPlantingData.SetSpacingDistance(fSpace);
      assertEquals(fSpace, m_oPlantingData.GetSpacingDistance(), 0);

      //Valid value
      fSpace = (float) 24.645;
      m_oPlantingData.SetSpacingDistance(fSpace);
      assertEquals(fSpace, m_oPlantingData.GetSpacingDistance(), 0);

    }
    catch (ModelException oErr) {
      fail("Normal processing for SetSpacingDistance failed with message " +
           oErr.sMessage);
    }

    //Exception processing
    try {
      fSpace = (float) - 24.645;
      m_oPlantingData.SetSpacingDistance(fSpace);
      fail("SetSpacingDistance did not catch a timestep value of " +
           String.valueOf(fSpace));
    }
    catch (ModelException oErr) {
      System.out.println(
          "Exception processing succeeded for SetSpacingDistance.");
    }
  }

  public void testValidatePlant() {
    PlantingData oToTest = null;
    Plot oPlot = null;
    TreePopulation oPop = null;

    try {
      oPlot = new Plot(null);
      oPlot.SetOldFileXGridCellLength(10);
      oPlot.SetPlotXLength(100);
      oPlot.SetOldFileYGridCellLength(10);
      oPlot.SetPlotYLength(100);
      oPlot.SetNumberOfTimesteps(3);

      oPop = new TreePopulation(null);
      oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});

      //Validation passes - random spacing
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 77);
      oToTest.AddAbundance(1, 23);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.SetDensity(300);
      oToTest.ValidatePlant(oPop, oPlot);

      //Validation passes - gridded spacing
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 35);
      oToTest.AddAbundance(1, 65);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.SetSpacingDistance( (float) 4.3);
      oToTest.ValidatePlant(oPop, oPlot);
    }
    catch (ModelException oErr) {
      fail("Normal processing failed for validation with message " +
           oErr.sMessage);
    }

    //Exception processing
    //No species
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 12);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Invalid species
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(2);
      oToTest.AddAbundance(0, 12);
      oToTest.AddAbundance(2, 12);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch invalid species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No planting specified specified
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddSpecies(0);
      oToTest.AddAbundance(0, 12);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of cut amount type.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No timestep specified
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 12);
      oToTest.AddSpecies(0);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of timesteps.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No cells
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddAbundance(0, 12);
      oToTest.AddSpecies(0);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of cells.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No abundance
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.SetTimestep(1);
      oToTest.AddSpecies(0);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of abundance.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Not abundances for all species
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, (float) 0.35);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of abundance for each species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Gridded abundances are not expressed as percentages - GRIDDED
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 135);
      oToTest.AddAbundance(1, 123);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of abundance for each species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Gridded abundances are not expressed as percentages - RANDOM
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 135);
      oToTest.AddAbundance(1, 123);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of abundance for each species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Gridded abundances don't add up to 100
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 35);
      oToTest.AddAbundance(1, 23);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of abundance for each species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Random abundances don't add up to 100
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 35);
      oToTest.AddAbundance(1, 23);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of abundance for each species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No spacing distance for gridded
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 35);
      oToTest.AddAbundance(1, 65);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of plant distance for gridded.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No density distance for random
    try {
      oToTest = null;
      oToTest = new PlantingData();
      oToTest.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddAbundance(0, 35);
      oToTest.AddAbundance(1, 65);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(1);
      oToTest.ValidatePlant(oPop, oPlot);
      fail("Validation did not catch lack of density for random.");
    }
    catch (ModelException oErr) {
      ;
    }
  }
}
